clear;
tic

iterations = 1;        %# of iterations performed for each Eb/N0 value   - one minute
K = 1024;                  %# of info bits

EbN0_db_vector = 0:.5:1;       %Range of Eb/N0 values   -> N_0 = 2 throughout
turbo_iterations = 5;   %# of iterations performed by turbo decoder

%define vector to be used for interleaving
for m = 0:(K-1)
    cm(m+1)=mod(m*(m+1)/2, K);
end;

outer_its = 1;
for EbN0_db = EbN0_db_vector
    
    EbN0 = 10^(EbN0_db/10)        %convert to linear units
    Eb = 2* EbN0;                 %set N_0 = 2  (different from Viterbi code!)
    sigma_squared = 1;            %because N_0 = 2
    Es = Eb/3;                    %because R=1/3, i.e., a third of an info bit per symbol
    
    map_error = zeros(1,turbo_iterations);  %initialiye error counter
    
    for its = 1:iterations
        
        %generate K random information bits
        pure_bits = round(rand(1,K));
        
        %encode bits
        [info_bits1 parity1] = encode_75_recursive_function(pure_bits);
        
        %interleave info bits
        for m = 1:(K-1)
            permuted_bits(cm(m)+1)=pure_bits(cm(m+1)+1);
        end;
        permuted_bits(cm(K)+1)=pure_bits(cm(1)+1);
        
        %if you wanted to de-interleave permuted_bits, you would run the
        %following commands:
        %for m = 1:(K-1)
        %    unpermuted_bits(cm(m+1)+1) = permuted_bits(cm(m)+1);
        %end;
        %unpermuted_bits(cm(1)+1)=permuted_bits(cm(K)+1);
        % These commands would result in unpermuted_bits == pure_bits
        
        %pass interleaved bits to encoder 2
        [info_bits2 parity2] = encode_75_recursive_function(permuted_bits);
        
        %define RX symbols for info bits1
        y_info1 = sqrt(Es) * (-2*info_bits1+1) + randn(1,K+2);
        
        %RX sequence for DEC2 is permuted version of RX sequence for DEC1
        for m = 1:(K-1)
            y_info2(cm(m)+1)=y_info1(cm(m+1)+1);
        end;
        y_info2(cm(K)+1)=y_info1(cm(1)+1);
        %different terminating bits for second encoder -only affects last 2
        %symbols
        y_info2(K+1:K+2) =  sqrt(Es) * (-2*info_bits2(K+1:K+2)+1) + randn(1,2);
        
        % NOTE: the vector of noisy information bits passed to DEC1 will be
        % y_info1, while the vector of noisy information bits passed to DEC2 will be
        % y_info2  (which is a permuted version of y_info1, and with
        % different symbols corresponding to the terminating bits)
        
        
        %RX symbols for different parity bits
        y_parity1 = sqrt(Es) * (-2*parity1+1) + randn(1,K+2);    %corresponds to y(2) in lecture
        y_parity2 = sqrt(Es) * (-2*parity2+1) + randn(1,K+2);    %corresponds to y(3) in lecture
        
        
        %initialize
        
        for turbo_its = 1:turbo_iterations
            
            %fill in code
            
            
            
            %current bit estimates based on decoder 2's output LLR's (LLR2)
            map_estimate2 = (LLR2 <0);
            
            %count bit errors
            map_error(turbo_its) = map_error(turbo_its) + sum(rem(map_estimate2+pure_bits,2));
            
        end;
        
    end;
    prob_bit_map(outer_its,:) = map_error / (K*iterations)
    outer_its = outer_its + 1;
end;

%Q(y) = .5 * erfc(y/sqrt(2))
EbN0_db_vector2 = min(EbN0_db_vector):3;
%uncoded bit error
uncoded_bit = .5*erfc(sqrt(10.^(EbN0_db_vector2/10)));

semilogy(EbN0_db_vector, prob_bit_map);
hold on;
semilogy(EbN0_db_vector2, uncoded_bit, 'b--');
hold on;
legend ('Bit Erorr', 'Bit - Uncoded');
grid on;

[EbN0_db_vector' prob_bit_map]

toc